@@ -6,7 +6,8 @@
 
 # Improvements: Set the corresponding threshold for each selected bandwidth, i.e. fix the bandwidth if the change of the selected bandwidths is less than threshold
 gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, criterion="dCVR", max.iterations=2000,threshold=0.00001, dMats, p.vals, theta.vals,longlat=FALSE,
-                     bws0, bw.seled=rep(F, length(bws0)), approach = "AIC", bws.thresholds=rep(0.1, length(dMats)), bws.reOpts=5,verbose=F,hatmatrix=T, nlower = 10)
+                           bws0, bw.seled=rep(F, length(bws0)), approach = "AIC", bws.thresholds=rep(0.1, length(dMats)), bws.reOpts=5, verbose=F, hatmatrix=T, 
+                           predictor.centered=rep(T, length(bws0)-1),nlower = 10)
 {
   ##Record the start time
   timings <- list()
@@ -26,78 +27,139 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
   {
     stop("Given regression data must be Spatial*DataFrame")
   }
-    #########Distance matrix is given or not
+  #########Distance matrix is given or not
   dp.n <- nrow(dp.locat)
   ######Extract the data frame
-   mf <- match.call(expand.dots = FALSE)
-    m <- match(c("formula", "data"), names(mf), 0L)
-    mf <- mf[c(1L, m)]
-    mf$drop.unused.levels <- TRUE
-    mf[[1L]] <- as.name("model.frame")
-    mf <- eval(mf, parent.frame())
-    mt <- attr(mf, "terms")
-    y <- model.extract(mf, "response")
-    x <- model.matrix(mt, mf)
-    idx1 <- match("(Intercept)", colnames(x))
-    if(!is.na(idx1))
-      colnames(x)[idx1]<-"Intercept" 
-    ##################################################
-    #####Linear regression
-    lms <- lm(formula,data=data)
-    #lms <-fastLm(formula,data=data) 
-    lms$x <- x
-    lms$y <- y
-    #colnames(x)[1]<-"Intercept"
-  ###check the distance matrices and bandwidths
+  mf <- match.call(expand.dots = FALSE)
+  m <- match(c("formula", "data"), names(mf), 0L)
+  mf <- mf[c(1L, m)]
+  mf$drop.unused.levels <- TRUE
+  mf[[1L]] <- as.name("model.frame")
+  mf <- eval(mf, parent.frame())
+  mt <- attr(mf, "terms")
+  y <- model.extract(mf, "response")
+  x <- model.matrix(mt, mf)
   var.n <- ncol(x)
+  #x2 <- x
+  idx1 <- match("(Intercept)", colnames(x))
+  ##################################################
+  #####Linear regression
+  lms <- lm(formula,data=data)
+  #lms <-fastLm(formula,data=data) 
+  lms$x <- x
+  lms$y <- y
+  #colnames(x)[1]<-"Intercept"
+  #################################################
+  ##########Centering and scaling predictors
+  if(!is.na(idx1))
+  {
+    if(length(predictor.centered)!=var.n-1)
+    {
+      predictor.centered <- rep(T, length.out=var.n-1)
+      warning("All the predictors will be centered, please check the parameter predictor.centered")
+    }
+    # if(length(predictor.scaled)!=var.n-1)
+    # {
+    #   predictor.scaled <- rep(F, length.out=var.n-1)
+    #   warning("All the predictors will not be scaled, please check the parameter predictor.scaled")
+    # }
+  }
+  else
+  {
+    if(length(predictor.centered)!=var.n)
+    {
+      predictor.centered <- rep(T, length.out=var.n)
+      warning("All the predictors will be centered, please check the parameter predictor.centered")
+    }
+    # if(length(predictor.scaled)!=var.n-1)
+    # {
+    #   predictor.scaled <- rep(F, length.out=var.n-1)
+    #   warning("All the predictors will not be scaled, please check the parameter predictor.scaled")
+    # }
+  }
+  
+  n.cent <- length(which(predictor.centered))
+  if(!is.na(idx1))
+  {
+    colnames(x)[idx1]<-"Intercept"
+    x1 <- x[,-idx1]
+  }
+  else
+  {
+    x1 <- x
+  }
+  #n.scal <- length(which(predictor.centered))
+  if(n.cent>1)
+    predictors.centered.means <- colMeans(x1[,predictor.centered])
+  else 
+    predictors.centered.means <- mean(x1[,predictor.centered])
+  if(n.cent>=1)
+  {
+    #meancent <- partial(scale, scale=FALSE)
+    x1[,predictor.centered] <- scale(x1[,predictor.centered], scale=FALSE)
+    
+  }
+  if(!is.na(idx1))
+  {
+    x1 <- cbind(1, x1)
+  }
+  colnames(x1) <-  colnames(x)
+  
+  #####Centering predictors
+  #####Centered predictors x1
+  #################################################
+  
+  ##Calculate the initial bandwidths: find an optimum bandwidth for each univariate model (dependent variable~each independent variable)
+  allvars <- all.vars(formula)
+  DeVar <- allvars[1]
+  InDevars <- colnames(x)
+  ##########################Check the distance matrices
+  ###check the distance matrices and bandwidths
   if(missing(dMats))
   {
     dMats <- list()
     if(missing(p.vals))
-	{
-	  p.vals <- 2
-	  dMat <- gw.dist(dp.locat,longlat=longlat)
-	  for(i in 1:var.n)
-	    dMats[[i]] <- dMat
-	}
-	else
-	{
-	  p.vals <- rep_len(p.vals, var.n)
-	  if(missing(theta.vals))
-	     theta.vals <- rep_len(0, var.n)
-	  else
-	     theta.vals<- rep_len(theta.vals, var.n)
-	  for(i in 1:var.n)
-	  {
-	    dMats[[i]] <- gw.dist(dp.locat, p=p.vals[i], theta=theta.vals[i],longlat=longlat)
-	  }
-	}
+    {
+      p.vals <- 2
+      dMat <- gw.dist(dp.locat,longlat=longlat)
+      for(i in 1:var.n)
+        dMats[[i]] <- dMat
+    }
+    else
+    {
+      p.vals <- rep_len(p.vals, var.n)
+      if(missing(theta.vals))
+        theta.vals <- rep_len(0, var.n)
+      else
+        theta.vals<- rep_len(theta.vals, var.n)
+      for(i in 1:var.n)
+      {
+        dMats[[i]] <- gw.dist(dp.locat, p=p.vals[i], theta=theta.vals[i],longlat=longlat)
+      }
+    }
   }
   else if(is.list(dMats))
   {
-     if(length(dMats)!=var.n)
-       stop("Please specify a distance matrix for each independent variable!")
-     else
-     {
-       for(i in 1:var.n)
-       {
-          dim.dMat<-dim(dMats[[i]])
-          if (dim.dMat[1]!=dp.n||dim.dMat[2]!=dp.n)
-             stop("Dimensions of dMat are not correct")
-       }
-     }
+    if(length(dMats)!=var.n)
+      stop("Please specify a distance matrix for each independent variable!")
+    else
+    {
+      for(i in 1:var.n)
+      {
+        dim.dMat<-dim(dMats[[i]])
+        if (dim.dMat[1]!=dp.n||dim.dMat[2]!=dp.n)
+          stop("Dimensions of dMat are not correct")
+      }
+    }
   }
   else
   {
     stop("dMats are not correct")
   }  
-  ##Calculate the initial bandwidths: find an optimum bandwidth for each univariate model (dependent variable~each independent variable)
-  allvars <- all.vars(formula)
-  DeVar <- allvars[1]
-  InDevars <- colnames(x)
+  ############################### Intialize the bandwidth
   if(missing(bws0))
   {
-     bws0 <- numeric(var.n)
+    bws0 <- numeric(var.n)
     cat("------ Calculate the initial bandwidths for each independent variable ------\n")
     for(i in 1:var.n)
     {
@@ -117,10 +179,18 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
     }
     cat("------            The end for the initial selections              ------\n")
   }
+  else
+  {
+    bws0 <- rep(bws0, length.out=var.n)
+  }
+  #################
+  if(length(bw.seled)!=var.n)
+    bw.seled <- rep(F, length.out=var.n)
+
   cat("------   Calculate the initial beta0 from the above bandwidths    ------\n")
   #dMat <- gw.dist(dp.locat=dp.locat, p=2, theta=0, longlat=longlat)
   dMat <- dMats[[1]]
-  bw.int0 <- bw.gwr1(x, y, dp.locat,approach=approach,kernel=kernel,adaptive=adaptive,dMat,verbose=F, nlower = nlower)
+  bw.int0 <- bw.gwr1(x1, y, dp.locat,approach=approach,kernel=kernel,adaptive=adaptive,dMat,verbose=F, nlower = nlower)
   #betas <- gwr.q(x, y, dp.locat, adaptive=adaptive, bw=bw.int0, kernel=kernel,dMat=dMat)
   #######Re-initialize the betas by a simple back-fitting process
   #betas <- gwr.backfit(x, y, betas,dp.locat,dp.locat, FALSE,criterion, adaptive, bws0, kernel,dMats, max.iterations,threshold*100)
@@ -132,7 +202,7 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
     C <- array(dim=c(dp.n,var.n,dp.n))
   }
   
-  res <- gwr.q2(x, y, dp.locat, adaptive=adaptive, hatmatrix = hatmatrix,bw=bw.int0, kernel=kernel,dMat=dMat)
+  res <- gwr.q2(x1, y, dp.locat, adaptive=adaptive, hatmatrix = hatmatrix,bw=bw.int0, kernel=kernel,dMat=dMat)
   betas <- res[[1]]
   if(hatmatrix)
   {
@@ -140,12 +210,11 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
     C <- res[[3]]
     idm <- diag(var.n)
     for(i in 1:var.n)
-       for(j in 1:dp.n)
+      for(j in 1:dp.n)
       {
-        S.arrays[i,j,] <- x[j,i]*(idm[i,]%*%C[j,,])
+        S.arrays[i,j,] <- x1[j,i]*(idm[i,]%*%C[j,,])
       }
   }
-  
   #betas <- gwr.backfit(x, y, betas,dp.locat,dp.locat, FALSE,criterion, adaptive, bws0, kernel,dMats, max.iterations,0.0001)
   cat("------            The end for calculating the initial beta0              ------\n") 
   cat("------ Select the optimum bandwidths for each independent variable via ", approach, " aproach ------\n")
@@ -156,186 +225,153 @@ gwr.multiscale <- function(formula, data, kernel="bisquare", adaptive=FALSE, cri
   criterion.val <- 10000000  
   #yhat.i <- betas*x
   #print(yhat.i)
-  resid.i <- y - gw.fitted(x, betas)
+  resid.i <- y - gw.fitted(x1, betas)
   RSS0 <- sum(resid.i^2)
   RSS1 <- 0
   RSS.vals <- c(RSS0, RSS1, criterion.val)
   cat("*****  The back-fitting process for model calibration with bandwiths selected *****\n")
   while((ieration < max.iterations) && criterion.val > threshold)
   { 
-     #AICcs <- numeric(var.n)
-     cat("    Iteration ", ieration+1, ":\n")   
-     for(i in 1:var.n)
-     {
-       dMat <- dMats[[i]]  
-       f.i <- betas[,i]*x[,i]
-       y.i <- resid.i + f.i
-       if(bw.seled[i])
-          bw.i <- bws0[i]
-       else
-       {
-          cat("Now select an optimum bandwidth for the variable: ", InDevars[i], "\n")
-          bw.i <- bw.gwr2(matrix(x[, i], ncol = 1), y.i, dp.locat, approach = approach, kernel = kernel, adaptive = adaptive, dMat, verbose = T, nlower = nlower)
-          cat("The newly selected bandwidth for variable ", InDevars[i])
-          cat(" is: ", bw.i, "\n")
-          cat("The bandwidth used in the last ieration is: ", bws0[i])
-           cat(" and the difference between these two bandwidths is: ", abs(bw.i - bws0[i]), "\n")
-           if (abs(bw.i - bws0[i]) > bws.thresholds[i]) {
-               cat("The bandwidth for variable ", InDevars[i])
-               cat(" will be continually selected in the next ieration.\n")
-               bws.change.NO[i] <- 0
-           }
-           else {
-               bws.change.NO[i] <- bws.change.NO[i] + 1
-               if(bws.change.NO[i] < bws.reOpts)
-               {
-                 cat("The bandwidth for variable ", InDevars[i])
-                 cat(" seems to be converged for ", bws.change.NO[i], " times.")
-                 cat("It will be continually optimized in the next ", bws.reOpts-bws.change.NO[i], " times\n")
-               }
-               else
-               {
-                 cat("The bandwidth for variable ", InDevars[i])
-                 cat(" seems to be converged and will be kept the same in the following ierations.\n")
-                 bw.seled[i] <- T
-               }
-           }
-       }
-       bws0[i] <- bw.i       
-       res <- gwr.q2(matrix(x[,i], ncol=1), y.i, dp.locat, adaptive=adaptive, hatmatrix = hatmatrix,bw=bw.i, kernel=kernel,dMat=dMat)
-       betai <- res[[1]]
-       if(hatmatrix)
-       {
-         Si <- res[[2]]
-         ###See Yu et al. 2018, eq. 18 on P8
-         S.arrayi <- S.arrays[i,,]
-         S.arrays[i,,] <- Si%*%S.arrayi + Si - Si%*%Shat
-         Shat <- Shat- S.arrayi + S.arrays[i,,] 
-       }
-       
-       #betai <- gwr.q(matrix(x[,i], ncol=1), y.i, loc=dp.locat, adaptive=adaptive, bw=bw.i, kernel=kernel,dMat=dMat)
-       #AICcs[i] <- gwr.aic(bw.i, matrix(x[,i], ncol=1), y.i, kernel, adaptive, dp.locat, dMat=dMat, verbose=F)
-       #yhat.i[,i] <- betai*x[,i]
-       betas[,i] <- betai
-       resid.i <- y - gw.fitted(x, betas)
-       #resid.i <- ehat(y.i, matrix(x[,i], ncol=1), matrix(betas[,i], ncol=1))
-       #betas[,i] <- betai
-     }
-     bws.vars <- rbind(bws.vars, bws0)
-     #AICc.vals <- rbind(AICc.vals, AICcs) 
-     RSS1 <- sum((y - gw.fitted(x, betas))^2)   
-     if(criterion=="CVR")
-     {
-         criterion.val <- abs(RSS1-RSS0)
-         cat("    Ieration ", ieration, "the change value of RSS (CVR) is: ", criterion.val,"\n")
-     }
-     else
-     {
-          criterion.val <- sqrt(abs(RSS1-RSS0)/RSS1)
-          cat("    Ieration ", ieration+1, "the differential change value of RSS (dCVR) is: ", criterion.val,"\n") 
-     }
-     RSS0 <- RSS1  
-     cat("----------End of    Iteration ", ieration+1, "----------\n") 
-     RSS.vals <- rbind(RSS.vals, c(RSS0, RSS1, criterion.val)) 
-     ieration <- ieration+1            
-   }
-   
-   #######Diagnostic information
-   #AICc.vals <- numeric(var.n)
-   #betas.SE <- betas
-   #betas.TV <- betas
-   
-   #S<-matrix(nrow=dp.n,ncol=dp.n)
-#   for(i in 1:var.n)
-#   {
-#     betai <- betas[,i]
-#     f.i <- betas[,i]*x[,i]
-#     y.i <- resid.i + f.i
-#     dMat <- dMats[[i]] 
-#     for(j in 1:dp.n)
-#     {
-#        W.i<-gw.weight(dMat[,j],bws0[i],kernel,adaptive)
-#        gwsi<-try(gw_reg(matrix(x[,i], ncol=1),y.i,W.i,T,j))
-#        if(!inherits(gwsi, "try-error"))
-#        {
-#           betai[j] <- gwsi[[1]]
-#           S[j,]<-gwsi[[2]]
-#           Ci<-gwsi[[3]]
-#           betas.SE[j,i]<-diag(Ci%*%t(Ci))
-#        }
-#        else
-#           break        
-#     }  
-#     tr.S<-sum(diag(S))
-#     tr.StS<-sum(S^2)
-#     Q<-t(diag(dp.n)-S)%*%(diag(dp.n)-S)
-#     RSS.gw<-t(y.i)%*%Q%*%y.i
-#     
-#     sigma.hat1<-RSS.gw/(dp.n-2*tr.S+tr.StS)
-#     sigma.hat2 <- RSS.gw/dp.n
-#     betas.SE[,i]<-sqrt(sigma.hat1*betas.SE[,i])
-#     betas.TV[,i] <- betai/betas.SE[,i]
-#     AICc.vals[i] <- dp.n*log(sigma.hat2) + dp.n*log(2*pi) + dp.n *((dp.n + tr.S) / (dp.n - 2 - tr.S))
-#     #R2.vals[i] <- 1-RSS.gwi/yss.gi
-#     #edf<- dp.n - 2*tr.S + tr.StS
-#     #R2adj.vals[i] <- 1-(1-R2.vals[i])*(dp.n-1)/(edf-1)
-#     betas[,i] <- betai
-#     resid.i <- y - gw.fitted(x, betas) 
-#   }
-   #Q<-t(diag(dp.n)-Shat)%*%(diag(dp.n)-Shat) 
-   #RSS.gw<-t(y)%*%Q%*%y 
-   RSS.gw <- RSS1
-   sigma.hat21 <- RSS.gw/dp.n
-   yss.g <- sum((y - mean(y))^2)
-   R2.val <-  1-RSS.gw/yss.g
-   R2adj <- NA
-   AICc <- NA
-   if(hatmatrix)
-   {
-     tr.Shat <- sum(diag(Shat))
-     tr.StShat<-sum(Shat^2)
-     edf<- dp.n - 2*tr.Shat + tr.StShat
-     AICc <- dp.n*log(sigma.hat21) + dp.n*log(2*pi) + dp.n *((dp.n + tr.Shat) / (dp.n - 2 - tr.Shat))
-     R2adj <- 1-(1-R2.val)*(dp.n-1)/(edf-1)
-   }
-   
-   #sigma.hat11<-RSS.gw/(dp.n-2*tr.Shat+tr.StShat)
-    GW.diagnostic<-list(RSS.gw=RSS.gw,AICc=AICc,R2.val=R2.val, R2adj = R2adj)
-    #####Output
-    yhat <- gw.fitted(x, betas)
-    residual <- y - yhat
-    vdgwr.df <- data.frame(betas, yhat, residual)
-    #colnames(vdgwr.df) <- c(colnames(x), "yhat", "residual",paste(colnames(betas), "SE", sep="_"), paste(colnames(betas), "TV", sep="_"))
-    colnames(vdgwr.df) <- c(colnames(x), "yhat", "residual")
-     griddedObj <- F
-    if (is(regression.points, "Spatial"))
-    { 
-        if (is(regression.points, "SpatialPolygonsDataFrame"))
-        {
-           polygons<-polygons(regression.points)
-           #SpatialPolygons(regression.points)
-           #rownames(gwres.df) <- sapply(slot(polygons, "polygons"),
-                              #  function(i) slot(i, "ID"))
-           SDF <-SpatialPolygonsDataFrame(Sr=polygons, data=vdgwr.df,match.ID=F)
+    #AICcs <- numeric(var.n)
+    cat("    Iteration ", ieration+1, ":\n")   
+    for(i in 1:var.n)
+    {
+      dMat <- dMats[[i]]  
+      f.i <- betas[,i]*x1[,i]
+      y.i <- resid.i + f.i
+      if(bw.seled[i])
+        bw.i <- bws0[i]
+      else
+      {
+        cat("Now select an optimum bandwidth for the variable: ", InDevars[i], "\n")
+        bw.i <- bw.gwr1(matrix(x1[, i], ncol = 1), y.i, dp.locat, approach = approach, kernel = kernel, adaptive = adaptive, dMat, verbose = T, nlower = nlower)
+        cat("The newly selected bandwidth for variable ", InDevars[i])
+        cat(" is: ", bw.i, "\n")
+        cat("The bandwidth used in the last ieration is: ", bws0[i])
+        cat(" and the difference between these two bandwidths is: ", abs(bw.i - bws0[i]), "\n")
+        if (abs(bw.i - bws0[i]) > bws.thresholds[i]) {
+          cat("The bandwidth for variable ", InDevars[i])
+          cat(" will be continually selected in the next ieration.\n")
+          bws.change.NO[i] <- 0
         }
-        else
-        {
-           griddedObj <- gridded(regression.points)
-           SDF <- SpatialPointsDataFrame(coords=dp.locat, data=vdgwr.df, proj4string=CRS(p4s), match.ID=F)
-           gridded(SDF) <- griddedObj 
+        else {
+          bws.change.NO[i] <- bws.change.NO[i] + 1
+          if(bws.change.NO[i] < bws.reOpts)
+          {
+            cat("The bandwidth for variable ", InDevars[i])
+            cat(" seems to be converged for ", bws.change.NO[i], " times.")
+            cat("It will be continually optimized in the next ", bws.reOpts-bws.change.NO[i], " times\n")
+          }
+          else
+          {
+            cat("The bandwidth for variable ", InDevars[i])
+            cat(" seems to be converged and will be kept the same in the following ierations.\n")
+            bw.seled[i] <- T
+          }
         }
+      }
+      bws0[i] <- bw.i       
+      res <- gwr.q2(matrix(x1[,i], ncol=1), y.i, dp.locat, adaptive=adaptive, hatmatrix = hatmatrix,bw=bw.i, kernel=kernel,dMat=dMat)
+      betai <- res[[1]]
+      if(hatmatrix)
+      {
+        Si <- res[[2]]
+        ###See Yu et al. 2018, eq. 18 on P8
+        S.arrayi <- S.arrays[i,,]
+        S.arrays[i,,] <- Si%*%S.arrayi + Si - Si%*%Shat
+        Shat <- Shat- S.arrayi + S.arrays[i,,] 
+      }
+      
+      #betai <- gwr.q(matrix(x[,i], ncol=1), y.i, loc=dp.locat, adaptive=adaptive, bw=bw.i, kernel=kernel,dMat=dMat)
+      #AICcs[i] <- gwr.aic(bw.i, matrix(x[,i], ncol=1), y.i, kernel, adaptive, dp.locat, dMat=dMat, verbose=F)
+      #yhat.i[,i] <- betai*x[,i]
+      betas[,i] <- betai
+      resid.i <- y - gw.fitted(x1, betas)
+      #resid.i <- ehat(y.i, matrix(x[,i], ncol=1), matrix(betas[,i], ncol=1))
+      #betas[,i] <- betai
+    }
+    bws.vars <- rbind(bws.vars, bws0)
+    #AICc.vals <- rbind(AICc.vals, AICcs) 
+    RSS1 <- sum((y - gw.fitted(x1, betas))^2)   
+    if(criterion=="CVR")
+    {
+      criterion.val <- abs(RSS1-RSS0)
+      cat("    Ieration ", ieration, "the change value of RSS (CVR) is: ", criterion.val,"\n")
     }
     else
-        SDF <- SpatialPointsDataFrame(coords=dp.locat, data=vdgwr.df, proj4string=CRS(p4s), match.ID=F)  
-   #GW.arguments<-list(formula=formula,rp.given=T,hatmatrix=T,criterion="RSS",bws=bws0, kernel=kernel,adaptive=adaptive)  
-   timings[["stop"]] <- Sys.time()
-   GW.arguments<-list(formula=formula,criterion=criterion,bws=bws0, kernel=kernel,adaptive=adaptive, hatmatrix=hatmatrix)  
-   #res <- list(SDF=SDF, GW.arguments=GW.arguments,lm=lms,timings=timings,this.call=this.call)
-   #class(res) <- "psdmgwr"
-   #invisible(res)
-    res <- list(SDF=SDF,GW.arguments=GW.arguments,GW.diagnostic=GW.diagnostic,lm=lms, bws.vars,timings=timings,this.call=this.call)
-    class(res) <- "multiscalegwr"
-    invisible(res) 
+    {
+      criterion.val <- sqrt(abs(RSS1-RSS0)/RSS1)
+      cat("    Ieration ", ieration+1, "the differential change value of RSS (dCVR) is: ", criterion.val,"\n") 
+    }
+    RSS0 <- RSS1  
+    cat("----------End of    Iteration ", ieration+1, "----------\n") 
+    RSS.vals <- rbind(RSS.vals, c(RSS0, RSS1, criterion.val)) 
+    ieration <- ieration+1            
+  }
+  #####Output
+  yhat <- gw.fitted(x1, betas)
+  residual <- y - yhat
+  ################
+  RSS.gw <- RSS1
+  sigma.hat21 <- RSS.gw/dp.n
+  yss.g <- sum((y - mean(y))^2)
+  R2.val <-  1-RSS.gw/yss.g
+  R2adj <- NA
+  AICc <- NA
+  if(hatmatrix)
+  {
+    tr.Shat <- sum(diag(Shat))
+    tr.StShat<-sum(Shat^2)
+    edf<- dp.n - 2*tr.Shat + tr.StShat
+    AICc <- dp.n*log(sigma.hat21) + dp.n*log(2*pi) + dp.n *((dp.n + tr.Shat) / (dp.n - 2 - tr.Shat))
+    R2adj <- 1-(1-R2.val)*(dp.n-1)/(edf-1)
+  }
+  
+  #sigma.hat11<-RSS.gw/(dp.n-2*tr.Shat+tr.StShat)
+  GW.diagnostic<-list(RSS.gw=RSS.gw,AICc=AICc,R2.val=R2.val, R2adj = R2adj)
+  #########################Retrive the intercept
+  if(!is.na(idx1) && n.cent>=1)
+  {
+    idx.cent <- which(predictor.centered)
+    betas.cent <- c()
+    for(i in 1:n.cent)
+      betas.cent <- cbind(betas.cent, betas[,idx.cent[i]+1]*predictors.centered.means[i])
+    beta0 <- betas[,idx1] - apply(betas.cent,1, sum)
+    betas[,idx1] <- beta0
+  }
+  ############################
+  vdgwr.df <- data.frame(betas, yhat, residual)
+  #colnames(vdgwr.df) <- c(colnames(x), "yhat", "residual",paste(colnames(betas), "SE", sep="_"), paste(colnames(betas), "TV", sep="_"))
+  colnames(vdgwr.df) <- c(colnames(x), "yhat", "residual")
+  griddedObj <- F
+  if (is(regression.points, "Spatial"))
+  { 
+    if (is(regression.points, "SpatialPolygonsDataFrame"))
+    {
+      polygons<-polygons(regression.points)
+      #SpatialPolygons(regression.points)
+      #rownames(gwres.df) <- sapply(slot(polygons, "polygons"),
+      #  function(i) slot(i, "ID"))
+      SDF <-SpatialPolygonsDataFrame(Sr=polygons, data=vdgwr.df,match.ID=F)
+    }
+    else
+    {
+      griddedObj <- gridded(regression.points)
+      SDF <- SpatialPointsDataFrame(coords=dp.locat, data=vdgwr.df, proj4string=CRS(p4s), match.ID=F)
+      gridded(SDF) <- griddedObj 
+    }
+  }
+  else
+    SDF <- SpatialPointsDataFrame(coords=dp.locat, data=vdgwr.df, proj4string=CRS(p4s), match.ID=F)  
+  #GW.arguments<-list(formula=formula,rp.given=T,hatmatrix=T,criterion="RSS",bws=bws0, kernel=kernel,adaptive=adaptive)  
+  timings[["stop"]] <- Sys.time()
+  GW.arguments<-list(formula=formula,criterion=criterion,bws=bws0, kernel=kernel,adaptive=adaptive, hatmatrix=hatmatrix)  
+  #res <- list(SDF=SDF, GW.arguments=GW.arguments,lm=lms,timings=timings,this.call=this.call)
+  #class(res) <- "psdmgwr"
+  #invisible(res)
+  res <- list(SDF=SDF,GW.arguments=GW.arguments,GW.diagnostic=GW.diagnostic,lm=lms, bws.vars,timings=timings,this.call=this.call)
+  class(res) <- "multiscalegwr"
+  invisible(res) 
 }
 
 ############################Layout function for outputing the Multiscale(PSDM) GWR results
@@ -464,29 +500,6 @@ gwr.backfit <- function(x, y, betas,dp.locat,rp.locat,hatmatrix, criterion="CVR"
    #calculate the hatmatrix
    betas
 }
-###For bandwidth selection
-bw.gwr2<-function(x, y, dp.locat,approach="AIC",kernel="gaussian",adaptive=FALSE,dMat, verbose=F, nlower = 10)
-{
-  dp.n <-  nrow(dp.locat)
-  if(adaptive)
-  {
-    upper <- dp.n
-    lower <- nlower
-  }
-  else
-  {
-      upper<-range(dMat)[2]
-      lower<-upper/5000
-  }
-  ########################## Now the problem for the golden selection is too computationally heavy
-    #Select the bandwidth by golden selection
-    bw<-NA
-    if(approach=="cv"||approach=="CV")
-       bw <- gold(gwr.cv,lower,upper,adapt.bw=adaptive,x,y,kernel,adaptive, dp.locat,dMat=dMat,verbose=verbose)
-    else if(approach=="aic"||approach=="AIC"||approach=="AICc")
-       bw <- gold(gwr.aic1,lower,upper,adapt.bw=adaptive,x,y,kernel,adaptive, dp.locat,dMat=dMat,verbose=verbose)
-    bw
-}
 gwr.cv2 <- function(bw, X, Y, kernel,adaptive, dp.locat,dMat)
 {
    dp.n<-length(dp.locat[,1])
